****************************************************************************
****************************************************************************
**                                                               
**  Phoenix (Basic "library" routines)
**
**  This software is in the public domain.  There is no warranty.
**
**  by Patrick Davidson (pad@calc.org, http://pad.calc.org/)
**
**  Last updated August 18, 2000
**
****************************************************************************
****************************************************************************

******************************************** TI-OS KEYBOARD READING

    IFND _nostub
Get_Key:
        lea     tios::kb_vars+$1c,a3    
        tst.w   (a3)                    ; See if a keypress is stored
        beq.s   Get_Key                 ; Continue waiting if not
        clr.w   (a3)                    ; Clear keypress counter
        move.w  2(a3),d0                ; Read keypress
        rts     

Test_Key:
        lea     tios::kb_vars+$1c,a3    
        tst.w   (a3)    
        beq.s   no_key  
        clr.w   (a3)    
        move.w  2(a3),d0        
        rts     
no_key: moveq   #0,d0
        rts     
    ENDIF

    IFD _nostub
Get_Key:
        bsr.s   Test_Key
        beq.s   Get_Key
        rts

Test_Key:
        moveq   #6,d0
        trap    #9                      ; Get pointer to keyboard queue in A0

        move.l  a0,-(sp)
        pea     _string_buffer(a5)
        move.l  ($c8).w,a0
        add.l   #$3aa*4,a0
        move.l  (a0),a0
        jsr     (a0)                    ; OSdequeue()
        addq.l  #8,sp
        tst.w   d0
        bne.s   no_key
        move.w  _string_buffer(a5),d0
        rts
no_key: moveq   #0,d0
        rts 
    ENDIF

******************************************** TI-89 KEYBOARD DELAY

    IFD ti89    
wait_keyport:
        moveq   #20,d1                  ; Increase this if there are
loop_keywait:
                                        ; keyboard reading errors.
        dbra    d1,loop_keywait 
        rts     
    ENDIF   

******************************************** RANDOM-NUMBER GENERATOR

random:                                 ; based on flib random
        move.l  d1,-(sp)        
        move.w  #0,d1   
        mulu.w  #31421,d1       
        add.w   #6927,d1        
        mulu.w  d1,d0   
        move.w  d1,random+4     
        clr.w   d0      
        swap    d0      
        move.l  (sp)+,d1        
        rts

******************************************** MISC. ROM FUNCTIONS

    IFND _nostub
____sprintf:
        jmp     tios::sprintf
ST_showHelp equ tios::ST_showHelp
    ENDIF
    IFD _nostub
____sprintf:
        move.l  ($c8).w,a0
        add.l   #$53*4,a0
        move.l  (a0),a0
        jmp     (a0)
ST_showHelp:
        move.l  ($c8).w,a0
        add.l   #$e6*4,a0
        move.l  (a0),a0
        jmp     (a0)
    ENDIF

******************************************** BASIC TEXT DISPLAY ROUTINES

Set_Font:                               ; Set font to D0
        move.w  d0,-(sp)        
    IFND _nostub
        jsr     tios::FontSetSys 
    ENDIF
    IFD _nostub
        move.l  ($c8).w,a0
        add.l   #$18f*4,a0
        move.l  (a0),a0
        jsr     (a0)
    ENDIF
        addq.w  #2,sp   
        rts     

Display_String:                         ; Display (A0) at (D1,D0)
        move.w  #4,-(sp)        
        lea     _unpack_buffer(a5),a1   ; A1 -> output string built here
        pea     (a1)
        move.w  d0,-(sp)        
        move.w  d1,-(sp)

loop_unpack_string:
        move.b  (a0)+,d0                ; Get new character of packed string
        beq.s   unpack_done             ; 0 = end of string
        bmi.s   decode_word             ; >= 128 --> unpack word
        move.b  d0,(a1)+                
        bra.s   loop_unpack_string
unpack_done:
        clr.b   (a1)

    IFND _nostub
        jsr     tios::DrawStrXY 
    ENDIF
    IFD _nostub
        move.l  ($c8).w,a0
        add.l   #$1a9*4,a0
        move.l  (a0),a0
        jsr     (a0)
    ENDIF
        lea     10(sp),sp       
        rts     

decode_word:
        move.l  a0,-(sp)
        lea     String_Patterns(pc),a0  ; A4 -> string pattern table
        bclr    #7,d0
locate_word_loop:
        subq.b  #1,d0                   ; subtract one from words to scan
        blt.s   copy_word_loop          ; < 128 --> done    
word_end_scan:
        tst.b   (a0)+                   ; Find end of string at (A4)
        bne.s   word_end_scan
        bra.s   locate_word_loop
copy_word_loop:                         ; Copy string from (A4) to (A1)
        move.b  (a0)+,(a1)+
        bne.s   copy_word_loop
        subq.l  #1,a1
        move.l  (sp)+,a0
        bra.s   loop_unpack_string
                                        
******************************************** STRING PATTERNS

spc     set     128

STRING  MACRO
\1      set     spc
spc     set     spc+1
        ENDM

String_Patterns:
        STRING  THE
        dc.b    ' the ',0
        STRING  WEAPON
        dc.b    ' weapon',0
        STRING  HOWBACK
    IFND ti89d   
        dc.b    'Press [SPACE] to return...',0  
    ENDIF   
    IFD ti89d   
        dc.b    'Press [ENTER] to return...',0  
    ENDIF
        STRING  DASH
        dc.b    ' - ',0
        STRING  YOU
        dc.b    ' you',0
        STRING  PHOENIX
        dc.b    'Phoenix',0
        STRING  CANNON
        dc.b    ' Cannon ',0
        STRING  PLASMA
        dc.b    ' Plasma Cannon ',0
    IFND ti89
        STRING  _AND
        dc.b    ' and ',0
        STRING  INFO
        dc.b    ' Info',0
    ENDIF
        STRING  UBLIC
        dc.b    'ublic',0
        STRING  OMAIN
        dc.b    'omain',0
        STRING  _TO
        dc.b    ' to ',0
        STRING  _IT
        dc.b    ' it.',0
        STRING  THIS
        dc.b    ' this',0

        EVEN
